import inspect
import os
from ..metisse import MetisseClass
import shutil


def get_current_path() -> str:
    """
    get current path
    """
    caller_frame = inspect.stack()[2]
    caller_file_path = caller_frame.filename
    _path = os.path.dirname(os.path.abspath(caller_file_path))
    return _path


def copy_images(src_folder, dest_folder, extensions=('jpg', 'jpeg', 'png', 'gif')):
    if not os.path.exists(dest_folder):
        os.makedirs(dest_folder)

    for root, _, files in os.walk(src_folder):
        for file in files:
            if file.lower().endswith(extensions):
                src_path = os.path.join(root, file)
                dest_path = os.path.join(dest_folder, file)
                shutil.copy(src_path, dest_path)


def copy_ui(src_folder, dest_folder, extensions='gui'):
    if not os.path.exists(dest_folder):
        os.makedirs(dest_folder)

    for root, _, files in os.walk(src_folder):
        for file in files:
            if extensions in file.lower():
                src_path = os.path.join(root, file)
                dest_path = os.path.join(dest_folder, file)
                shutil.copy(src_path, dest_path)


def example():
    print(get_current_path())


def create_example_py_file():
    _path = get_current_path()
    _tmp = MetisseClass('01234567(test_uid)', _path, None, 'android')
    _tmp._logger.close()
    _curPath = os.path.abspath(os.path.dirname(__file__))

    source_folder = os.path.join(_curPath, "example_data", 'icon')
    destination_folder = os.path.join(_path, 'icon', 'script_example')
    copy_images(source_folder, destination_folder)

    source_folder = os.path.join(_curPath, "example_data", 'temp_image')
    destination_folder = os.path.join(_path, '01234567(test_uid)', 'temp_image')
    copy_images(source_folder, destination_folder)

    source_folder = os.path.join(_curPath, "example_data", 'ui')
    destination_folder = os.path.join(_path, 'ui')
    copy_ui(source_folder, destination_folder)

    content = '''import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.metisse import MetisseClass
from metisse.params import ImageRecognitionParams , SaveParams
TEMPLATE = {'template_image_secondary_dir' : 'script_example'}
SAVE = {'save_image_secondary_dir' : 'script_example'}

class script_example(MetisseClass):

    def __init__(self, device_id='', relatively_path='', pyqt6_ui_label={}, os_environment=''):
        MetisseClass.__init__(
            self,
            device_id=device_id,
            relatively_path=relatively_path,
            pyqt6_ui_label=pyqt6_ui_label,
            os_environment=os_environment,
        )

    def __call__(self, *args, **kwargs):
        self.check_image_recognition(
            ImageRecognitionParams(screen_image_name='tmp0',
                                   template_image_name='example_template_hand',
                                   is_refresh_screenshot=False,**TEMPLATE))  # simulate image recognition hand icon
        print(f'hand pos = {self._img_recog_result.coordinate}')

        self.execute_time_sleep(1)  # wait 1 second

        self.default_tap(
            ImageRecognitionParams(screen_image_name='tmp0',
                                   template_image_name='example_template_face',
                                   template_image_secondary_dir='script_example',
                                   is_refresh_screenshot=False))  # simulate image recognition face icon and tap face pos
        self.default_tap(
            ImageRecognitionParams(screen_image_name='tmp0',
                                   template_image_name='example_template_face',
                                   is_refresh_screenshot=False,**TEMPLATE))  # simulate image recognition face icon and tap face pos

        self.save_screenshot_compression( SaveParams(
                       save_image_name='save_image',
                       save_image_primary_dir='storage',
                       save_image_secondary_dir='script_example',
                       compression=0.5,
                       screenshot_wait_time=1,
                       is_save_image_name_add_time=True,
                       is_refresh_screenshot=False,
                       ))
        self.save_screenshot_compression( SaveParams(
                       save_image_name='save_image',
                       save_image_primary_dir='storage',
                       compression=0.5,
                       screenshot_wait_time=1,
                       is_save_image_name_add_time=True,
                       is_refresh_screenshot=False,
                       **SAVE,
                       ))
        self.save_screenshot_compression( SaveParams(
                       save_image_name='save_image',
                       save_image_primary_dir='storage',
                       save_image_subdirs = ['third','forth'],
                       compression=0.5,
                       screenshot_wait_time=1,
                       is_save_image_name_add_time=True,
                       is_refresh_screenshot=False,
                       **SAVE,
                       ))
        self.crop_screenshot((288, 260), (340, 294),
                             SaveParams(load_image_primary_dir='temp_image',
                                        save_image_primary_dir='storage',
                                        save_image_name='eyes',
                                        load_image_name='tmp0',
                                        is_refresh_screenshot=False))
        self.crop_screenshot((288, 260), (340, 294),
                             SaveParams(load_image_primary_dir='temp_image',
                                        save_image_primary_dir='temp_image',
                                        save_image_secondary_dir='crop',
                                        save_image_name='small_image_tmp',
                                        load_image_name='tmp0',
                                        is_refresh_screenshot=False))
        self.default_tap(
            ImageRecognitionParams(screen_image_name='tmp0',
                                   template_image_name='small_image_tmp',
                                   template_image_primary_dir='temp_image',
                                   template_image_secondary_dir='crop',
                                   is_refresh_screenshot=False))  # simulate image recognition face icon and tap face pos


if __name__ == '__main__':
    script_obj = script_example('01234567(test_uid)', None, None, 'android')
    script_obj()
'''

    with open(os.path.join(_path, "script_example.py"), "w", encoding="utf-8") as file:
        file.write(content)



import os
import sys
import pytest
from unittest import mock
from unittest.mock import MagicMock

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.clients.ios.wda import WdaClient
from metisse.params import DeviceParams


@pytest.fixture
def ios_client():
    device_params = DeviceParams(
        device_id="test_virtual_device",
        os_environment="android",
    )
    device_params = DeviceParams(os_environment="ios", device_id="test_virtual_device")
    client = WdaClient(device_params, skip_wda_launch=True)
    return client



@mock.patch("metisse.clients.ios.wda.WdaClient.screenshot")
def test_screenshot(mock_screenshot,ios_client):
    client = ios_client
    save_screenshot_path = "test_screenshot.png"
    client.screenshot(save_screenshot_path)
    mock_screenshot.assert_called_once()

@mock.patch("metisse.clients.ios.wda.WdaClient.tap")
def test_tap(mock_tap,ios_client):
    client = ios_client
    coordinates = (100, 200)
    client.tap(coordinates)
    mock_tap.assert_called_once()

@mock.patch("metisse.clients.ios.wda.WdaClient.swipe")
def test_swipe(mock_swipe,ios_client):
    client = ios_client
    start_coordinates = (100, 200)
    end_coordinates = (300, 400)
    swiping_time = 1.0
    client.swipe(start_coordinates, end_coordinates, swiping_time)
    mock_swipe.assert_called_once()


if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_wdaclient.py'])


import os
import shutil
import sys
import tempfile
from unittest import mock
import pytest
from unittest.mock import patch
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from metisse.metisse import MetisseClass
from metisse.params import DeviceParams, UiClientParams
from metisse.clients.android import AdbClient
from metisse.clients.ios import WdaClient
from metisse.utils.metisse_path import DevPath
from metisse.utils.ui_client import UiClient
from PyQt6.QtWidgets import QApplication, QLabel
app = QApplication(sys.argv)
@pytest.fixture
def test_metisse_class_test_dir():
    test_dir = tempfile.mkdtemp()
    yield test_dir
    shutil.rmtree(test_dir)

def test_metisse_class_init_android(test_metisse_class_test_dir):
    test_dir = test_metisse_class_test_dir

    device_id = "test_virtual_device"
    relatively_path = test_dir
    pyqt6_ui_label = UiClient(UiClientParams( QLabel(), QLabel()))
    os_environment = "android"
    with patch("metisse.utils.metisse_log.MetisseLogger.__init__", return_value=None)as mock_metisse_log_init:
        metisse = MetisseClass(device_id, relatively_path, pyqt6_ui_label, os_environment)
        mock_metisse_log_init.assert_called_once()
        assert metisse._device_id == device_id
        assert metisse._relatively_path == test_dir
        assert isinstance(metisse._ui_client, UiClient)
        assert metisse._os_environment == os_environment
        assert isinstance(metisse._client, AdbClient)

def test_metisse_class_init_ios(test_metisse_class_test_dir):
    test_dir = test_metisse_class_test_dir
    # Test for iOS
    device_id = "test_virtual_device"
    relatively_path = test_dir
    pyqt6_ui_label = None
    os_environment_ios = "ios"
    with patch("metisse.utils.metisse_log.MetisseLogger.__init__", return_value=None)as             mock_metisse_log_init , \
        patch("metisse.clients.ios.wda.WdaClient.__init__", return_value=None) as mock_wda_client_init:
        metisse_ios = MetisseClass(device_id, relatively_path, pyqt6_ui_label, os_environment_ios)
        mock_wda_client_init.assert_called_once()
        mock_metisse_log_init.assert_called_once()
    assert metisse_ios._os_environment == os_environment_ios
    assert isinstance(metisse_ios._client, WdaClient)

def test_metisse_class_init_invalid_os(test_metisse_class_test_dir):
    test_dir = test_metisse_class_test_dir
    # Test for invalid os_environment
    device_id = "test_virtual_device"
    relatively_path = test_dir
    pyqt6_ui_label = None
    os_environment_invalid = "invalid"
    with patch("metisse.utils.metisse_log.MetisseLogger.__init__", return_value=None)as mock_metisse_log_init:
        with pytest.raises(AssertionError):
            MetisseClass(device_id, relatively_path, pyqt6_ui_label, os_environment_invalid)
            mock_metisse_log_init.assert_called_once()

if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_metisse_class.py'])


import shutil
import sys
import os
import tempfile
from unittest.mock import patch, MagicMock
import pytest
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
# Import other necessary modules and classes
from metisse.params import ImageRecognitionParams, UiClientParams
from metisse.metisse import MetisseClass


@pytest.fixture
def test_default_tap_setUp():
    test_dir = tempfile.mkdtemp()
    curPath = os.path.abspath(os.path.dirname(__file__))
    test_metisse = MetisseClass(
        device_id='test_virtual_device',
        relatively_path=test_dir,
        pyqt6_ui_label=UiClientParams(),
        os_environment='android',
    )
    test_metisse._logger.close()
    test_metisse.is_backup = True
    test_metisse.screenshot_wait_time_increase = 1
    test_metisse.is_check_gamelog = False

    src_test_images = os.path.join(curPath, 'test_data','image','test_template.png')
    dst_test_images = os.path.join(test_dir,'icon' ,'test_template.png')
    shutil.copy(src_test_images, dst_test_images)

    src_test_images = os.path.join(curPath, 'test_data','image','test_template_fail.png')
    dst_test_images = os.path.join(test_dir,'icon' ,'test_template_fail.png')
    shutil.copy(src_test_images, dst_test_images)

    src_test_images = os.path.join(curPath,'test_data','image','tmp0.png')
    dst_test_images = os.path.join(test_dir,'test_virtual_device','temp_image' ,'tmp0.png')
    shutil.copy(src_test_images, dst_test_images)

    yield test_metisse
    shutil.rmtree(test_dir)

@patch.object(MetisseClass, 'check_image_recognition', return_value=True)
@patch.object(MetisseClass, 'tap')
@patch.object(MetisseClass, 'save_screenshot_compression')
def test_adb_default_tap(mock_save_screenshot_compression, mock_tap, mock_check_image_recognition,test_default_tap_setUp):
    # Set up the necessary input parameters and objects
    params = ImageRecognitionParams(template_image_name="test_template",
                                    template_image_primary_dir="icon",
                                    is_backup=True)

    # Call the default_tap method with the prepared input parameters
    result = test_default_tap_setUp.default_tap(params)

    # Check if the mocked methods were called with the expected arguments
    mock_check_image_recognition.assert_called_with(params)
    mock_tap.assert_called_once()
    mock_save_screenshot_compression.assert_called_once()

    # Assert the return value of the default_tap method
    assert result, "check_image_recognition failed with basic params"



if __name__ == '__main__':
    pytest.main(['-v','-s','pytest_metisse/test_default_tap.py'])



# -*- coding=UTF-8 -*-
# pyright: strict

import cv2
from cv2 import Mat


class Opencv_utils(object):
    """
    image similarity calculation
    """

    def __init__(self, screen_image_path: str, template_image_path: str) -> None:
        self.screen_image_path = screen_image_path
        self.template_image_path = template_image_path
        self.screen_image_mat: Mat
        self.template_image_mat: Mat
        self._set_screen_image_imread_cv2()
        self._set_template_image_imread_cv2()

    def _set_screen_image_imread_cv2(self) -> None:
        self.screen_image_mat = cv2.imread(self.screen_image_path)

    def _set_template_image_imread_cv2(self) -> None:
        self.template_image_mat = cv2.imread(self.template_image_path)



# -*- coding=UTF-8 -*-
# pyright: strict
import inspect
import os
from ..settings import Settings as ST
from pathlib import Path
from typing import Any
from ..params import ImageRecognitionParams, SaveParams


class DevPath(object):
    """
    setup dev environment
    """

    def __init__(self, current_path: str) -> None:
        self._absolute_path = os.path.abspath(current_path)
        self.initialize_dev_environment()

    def initialize_dev_environment(self) -> None:
        """
        generate dev root path (icon , ui ) files
        """
        for _document_path in ST.DEV_ENVIRONMENT_ROOT_PATH:
            _document_path_temp = os.path.join(self._absolute_path, _document_path)
            if not os.path.isdir(_document_path_temp):
                os.makedirs(_document_path_temp)

    @staticmethod
    def auto_generate_dev_path():
        """
        auto generate dev root path
        """
        caller_frame = inspect.stack()[1]  # 获取调用栈的上一级帧
        caller_file_path = caller_frame.filename  # 获取调用该方法的文件路径

        _path = os.path.dirname(os.path.abspath(caller_file_path))
        _path = os.path.join(_path, ".")
        dev_path = DevPath(_path)
        dev_path.initialize_dev_environment()

    def create_extended_script_path(self, device_id: str) -> 'ScriptPath':
        """
        Create an ExtendedDevPath object that inherits from DevPath and has new attributes and methods.

        Args:
            new_attribute (str): The new attribute to be added to the ExtendedDevPath object.

        Returns:
            ExtendedDevPath: An instance of the ExtendedDevPath class.
        """
        return ScriptPath(self._absolute_path, device_id)


class ScriptPath(DevPath):
    """
    This class inherits from DevPath and adds new features.
    """

    def __init__(self, current_path: str, device_id: str) -> None:
        super().__init__(current_path)
        self.absolute_path = current_path
        self.device_id_path = os.path.abspath(os.path.join(
            current_path, device_id))  # specific device id path (init script auto generate device id file)
        self.device_id = device_id
        self.initialize_script_environment()

    def initialize_script_environment(self) -> None:
        """
        generate script root path (temp_image,storage,backup,log ) files
        """
        for _document_path in ST.SCRIPT_ENVIRONMENT_ROOT_PATH:
            _document_path_temp = os.path.join(self.device_id_path, _document_path)
            if not os.path.isdir(_document_path_temp):
                os.makedirs(_document_path_temp)

    @staticmethod
    def auto_generate_script_path(device_id: str) -> object:
        """
        auto generate script root path
        """
        caller_frame = inspect.stack()[1]  # 获取调用栈的上一级帧
        caller_file_path = caller_frame.filename  # 获取调用该方法的文件路径

        _path = os.path.dirname(os.path.abspath(caller_file_path))
        _path = os.path.join(_path, device_id)
        script_path = ScriptPath(device_id, _path)
        script_path.initialize_script_environment()
        return script_path

    def _check_image_name_pngFormat(self, _input_name: str) -> str:
        if '.png' in _input_name:
            return _input_name
        return _input_name + '.png'

    def check_path(self, _path: str) -> str:
        _tmp_check_path = Path(_path)
        _tmp_check_path.mkdir(parents=True, exist_ok=True)
        return str(_path)

    def get_image_path(self, _image_name: str = '', _image_root: str = '', _additional_root: str = '') -> str:
        _image_name = self._check_image_name_pngFormat(_image_name)
        self._screen_image_path = os.path.join(self.device_id_path, _image_root, _additional_root, _image_name)
        return self._screen_image_path

    def get_screen_image_path(self, path_params: ImageRecognitionParams) -> str:
        _image_name = self._check_image_name_pngFormat(path_params.screen_image_name)
        _subdirs = ''
        if path_params.screen_image_subdirs:
            _subdirs = os.path.join(*path_params.screen_image_subdirs)
        self._screen_image_path = os.path.join(self.device_id_path, path_params.screen_image_primary_dir,
                                               path_params.screen_image_secondary_dir, _subdirs, _image_name)
        return self._screen_image_path

    def get_template_image_path(self, path_params: ImageRecognitionParams) -> str:
        _image_name = self._check_image_name_pngFormat(path_params.template_image_name)
        _subdirs = ''
        if path_params.template_image_subdirs:
            _subdirs = os.path.join(*path_params.template_image_subdirs)
        if path_params.template_image_primary_dir == 'temp_image':  # temp_image default in the device folder , adjust path
            self._screen_image_path = os.path.join(self.device_id_path, path_params.template_image_primary_dir,
                                                   path_params.template_image_secondary_dir, _subdirs, _image_name)
        else:
            self._screen_image_path = os.path.join(self.absolute_path, path_params.template_image_primary_dir,
                                                   path_params.template_image_secondary_dir, _subdirs, _image_name)

        return self._screen_image_path

    def get_load_image_path(self, path_params: SaveParams) -> str:
        _image_name = self._check_image_name_pngFormat(path_params.load_image_name)
        _subdirs = ''
        if path_params.load_image_subdirs:
            _subdirs = os.path.join(*path_params.load_image_subdirs)
        self._screen_image_path = os.path.join(self.device_id_path, path_params.load_image_primary_dir,
                                               path_params.load_image_secondary_dir, _subdirs, _image_name)
        return self._screen_image_path

    def get_save_image_path(self, path_params: SaveParams) -> str:
        _image_name = self._check_image_name_pngFormat(path_params.save_image_name)
        _subdirs = ''
        if path_params.save_image_subdirs:
            _subdirs = os.path.join(*path_params.save_image_subdirs)

        self._screen_image_path = os.path.join(self.device_id_path, path_params.save_image_primary_dir,
                                               path_params.save_image_secondary_dir, _subdirs, _image_name)

        return self._screen_image_path


import sys
import pytest
import os
import shutil
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.example.generate_example import create_example_py_file

@pytest.fixture
def test_generate_example_setup():
    current_path = os.path.abspath(os.path.dirname(__file__))
    example_py_path = os.path.join(current_path, "script_example.py")
    icon_folder_path = os.path.join(current_path, 'icon')
    temp_image_folder_path = os.path.join(current_path, '01234567(test_uid)', 'temp_image')
    ui_folder_path = os.path.join(current_path, 'ui')
    device_folder_path = os.path.join(current_path, '01234567(test_uid)')
    yield example_py_path, icon_folder_path, temp_image_folder_path, ui_folder_path, device_folder_path
    if os.path.exists(example_py_path):
        os.remove(example_py_path)
    if os.path.exists(temp_image_folder_path):
        shutil.rmtree(temp_image_folder_path)
    if os.path.exists(icon_folder_path):
        shutil.rmtree(icon_folder_path)
    if os.path.exists(ui_folder_path):
        shutil.rmtree(ui_folder_path)
    if os.path.exists(device_folder_path):
        shutil.rmtree(device_folder_path)
        
def test_create_example_py_file(test_generate_example_setup):
    example_py_path, icon_folder_path, temp_image_folder_path, ui_folder_path, device_folder_path = test_generate_example_setup
    create_example_py_file()
    assert os.path.exists(example_py_path), "Example py file not created."
    assert os.path.exists(icon_folder_path), "Icon folder not created."
    assert os.path.exists(temp_image_folder_path), "Temp image folder not created."
    assert os.path.exists(ui_folder_path), "ui folder not created."

if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_generate_example.py'])


import sys
import time
import pytest
import tempfile
import os
import logging

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.utils.metisse_log import MetisseLogger



@pytest.fixture
def test_metis_log_setup():
    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        log_file = temp_file.name
        unique_id = int(time.time() * 1000)
        logger_name = "test_logger_{}".format(unique_id)
        metisse_logger = MetisseLogger(logger_name=logger_name, log_level=logging.DEBUG, log_file=log_file)
    yield metisse_logger, log_file
    metisse_logger.close()
    os.remove(log_file)

def test_log_file_creation(test_metis_log_setup):
    metisse_logger, log_file = test_metis_log_setup
    assert os.path.exists(log_file)

def test_log_debug_message(test_metis_log_setup):
    metisse_logger, log_file = test_metis_log_setup
    message = "Test debug message"
    metisse_logger.debug(message)
    _assert_log_contains(message, log_file)

def test_log_info_message(test_metis_log_setup):
    metisse_logger, log_file = test_metis_log_setup
    message = "Test info message"
    metisse_logger.info(message)
    _assert_log_contains(message, log_file)

def test_log_warning_message(test_metis_log_setup):
    metisse_logger, log_file = test_metis_log_setup
    message = "Test warning message"
    metisse_logger.warning(message)
    _assert_log_contains(message, log_file)

def test_log_error_message(test_metis_log_setup):
    metisse_logger, log_file = test_metis_log_setup
    message = "Test error message"
    metisse_logger.error(message)
    _assert_log_contains(message, log_file)

def test_log_critical_message(test_metis_log_setup):
    metisse_logger, log_file = test_metis_log_setup
    message = "Test critical message"
    metisse_logger.critical(message)
    _assert_log_contains(message, log_file)

def _assert_log_contains(message, log_file):
    with open(log_file, "r") as log_file:
        log_contents = log_file.read()
        assert message in log_contents


if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_metis_log.py'])


import sys
import tempfile
import os
import shutil
from unittest.mock import patch
import pytest

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.params import ImageRecognitionParams, UiClientParams
from metisse.metisse import MetisseClass


@pytest.fixture(scope='module')
def metisse_test_setup():
    test_dir = tempfile.mkdtemp()
    curPath = os.path.abspath(os.path.dirname(__file__))
    test_metisse = MetisseClass(
        device_id='test_virtual_device',
        relatively_path=test_dir,
        pyqt6_ui_label=UiClientParams(),
        os_environment='android',
    )
    test_metisse._logger.close()
    test_metisse.is_backup = False
    test_metisse.screenshot_wait_time_increase = 1
    test_metisse.is_check_gamelog = False

    src_test_images = os.path.join(curPath, 'test_data','image','test_template.png')
    dst_test_images = os.path.join(test_dir,'icon' ,'test_template.png')
    shutil.copy(src_test_images, dst_test_images)

    src_test_images = os.path.join(curPath, 'test_data','image','test_template_fail.png')
    dst_test_images = os.path.join(test_dir,'icon' ,'test_template_fail.png')
    shutil.copy(src_test_images, dst_test_images)

    src_test_images = os.path.join(curPath,'test_data','image','tmp0.png')
    dst_test_images = os.path.join(test_dir,'test_virtual_device','temp_image' ,'tmp0.png')
    shutil.copy(src_test_images, dst_test_images)

    yield test_metisse
    shutil.rmtree(test_dir)


def test_check_image_recognition_basic(metisse_test_setup):
    params = ImageRecognitionParams(
        template_image_name='test_template',
        template_image_primary_dir='icon',
    )
    with patch("metisse.metisse.MetisseClass.screenshot", return_value=None) as mock_screenshot:
        result = metisse_test_setup.check_image_recognition(params)
        mock_screenshot.assert_called_once()
    assert result, "check_image_recognition failed with basic params"


def test_check_image_recognition_specific_case(metisse_test_setup):
    params = ImageRecognitionParams(
        template_image_name='test_template_fail',
        template_image_primary_dir='icon',
    )

    with patch("metisse.metisse.MetisseClass.screenshot", return_value=None) as mock_screenshot:
        result = metisse_test_setup.check_image_recognition(params)
        mock_screenshot.assert_called_once()
    assert not result, "check_image_recognition failed with specific params"


if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_check_image_recognition.py'])


# Form implementation generated from reading ui file 'c:\Users\henrychen\Documents\gitlab\metisse\metisse\example\ui\universal_script_gui_sub2.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):

    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(463, 351)
        self.stop_Button = QtWidgets.QPushButton(Dialog)
        self.stop_Button.setGeometry(QtCore.QRect(150, 290, 171, 41))
        self.stop_Button.setObjectName("stop_Button")
        self.info_label = QtWidgets.QLabel(Dialog)
        self.info_label.setGeometry(QtCore.QRect(40, 20, 281, 111))
        self.info_label.setObjectName("info_label")
        self.size_listWidget = QtWidgets.QListWidget(Dialog)
        self.size_listWidget.setGeometry(QtCore.QRect(30, 140, 91, 81))
        self.size_listWidget.setObjectName("size_listWidget")
        self.adb_kill_button = QtWidgets.QPushButton(Dialog)
        self.adb_kill_button.setGeometry(QtCore.QRect(240, 230, 201, 41))
        self.adb_kill_button.setObjectName("adb_kill_button")
        self.adjust_button = QtWidgets.QPushButton(Dialog)
        self.adjust_button.setGeometry(QtCore.QRect(120, 140, 111, 41))
        self.adjust_button.setObjectName("adjust_button")
        self.reset_button = QtWidgets.QPushButton(Dialog)
        self.reset_button.setGeometry(QtCore.QRect(120, 180, 111, 41))
        self.reset_button.setObjectName("reset_button")
        self.adb_button = QtWidgets.QPushButton(Dialog)
        self.adb_button.setGeometry(QtCore.QRect(30, 230, 201, 41))
        self.adb_button.setObjectName("adb_button")
        self.dpi_listWidget = QtWidgets.QListWidget(Dialog)
        self.dpi_listWidget.setGeometry(QtCore.QRect(240, 140, 91, 81))
        self.dpi_listWidget.setObjectName("dpi_listWidget")
        self.dpi_adjust_button = QtWidgets.QPushButton(Dialog)
        self.dpi_adjust_button.setGeometry(QtCore.QRect(330, 140, 111, 41))
        self.dpi_adjust_button.setObjectName("dpi_adjust_button")
        self.dpi_reset_button = QtWidgets.QPushButton(Dialog)
        self.dpi_reset_button.setGeometry(QtCore.QRect(330, 180, 111, 41))
        self.dpi_reset_button.setObjectName("dpi_reset_button")
        self.get_device_info_Button = QtWidgets.QPushButton(Dialog)
        self.get_device_info_Button.setGeometry(QtCore.QRect(330, 30, 111, 81))
        self.get_device_info_Button.setObjectName("get_device_info_Button")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.stop_Button.setText(_translate("Dialog", "關閉"))
        self.info_label.setText(_translate("Dialog", "TextLabel"))
        self.adb_kill_button.setText(_translate("Dialog", "adb kill"))
        self.adjust_button.setText(_translate("Dialog", "調整解析度"))
        self.reset_button.setText(_translate("Dialog", "回復解析度"))
        self.adb_button.setText(_translate("Dialog", "adb connect"))
        self.dpi_adjust_button.setText(_translate("Dialog", "調整dpi"))
        self.dpi_reset_button.setText(_translate("Dialog", "回復dpi"))
        self.get_device_info_Button.setText(_translate("Dialog", "獲取當前裝置\n"
                                                       "螢幕解析度\n"
                                                       "dpi"))



import os
import shutil
import sys
import tempfile
import pytest
from PIL import Image

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.params import SaveParams, UiClientParams
from metisse.metisse import MetisseClass




@pytest.fixture
def test_crop_screenshot_setup():
    test_dir = tempfile.mkdtemp()
    curPath = os.path.abspath(os.path.dirname(__file__))
    test_metisse = MetisseClass(
        device_id='test_virtual_device',
        relatively_path=test_dir,
        pyqt6_ui_label=UiClientParams(),
        os_environment='android',
    )
    test_metisse._logger.close()
    test_metisse.is_backup = False
    test_metisse.screenshot_wait_time_increase = 1
    test_metisse.is_check_gamelog = False

    src_test_images = os.path.join(curPath,'test_data','image','tmp0.png')
    dst_test_images = os.path.join(test_dir,'test_virtual_device','temp_image' ,'tmp0.png')
    shutil.copy(src_test_images, dst_test_images)

    yield test_metisse
    shutil.rmtree(test_dir)

def test_crop_screenshot(test_crop_screenshot_setup):
    test_metisse = test_crop_screenshot_setup
    save_params = SaveParams(save_image_primary_dir='storage',
                                save_image_name='test_cropped_image',
                                save_image_secondary_dir='',
                                is_refresh_screenshot=False,
                                screenshot_wait_time=0.1,
                                is_save_image_name_add_time=False,
                                compression=1)

    coordinate1 = (100, 50)
    coordinate2 = (300, 400)

    test_metisse.crop_screenshot(coordinate1, coordinate2, save_params)

    expected_output_path = os.path.join(test_metisse._script_path.device_id_path, save_params.save_image_primary_dir,
                                        save_params.save_image_name)
    assert os.path.exists(expected_output_path) , 'Cropped image file was not created.'

    with Image.open(expected_output_path) as cropped_image:
        expected_size = (coordinate2[0] - coordinate1[0], coordinate2[1] - coordinate1[1])
        assert cropped_image.size == expected_size, 'Cropped image size does not match expected size.'




if __name__ == '__main__':
    pytest.main(['-v','-s','pytest_metisse/test_crop_screenshot.py'])



import os
import sys
import pytest
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from unittest.mock import patch
from metisse.clients.android.adb import AdbClient
from metisse.params import DeviceParams

@pytest.fixture
def adb_client():
    device_params = DeviceParams(
        device_id="test_virtual_device",
        os_environment="android",
    )
    return AdbClient(device_params)


def test_screenshot(adb_client):
    with patch("os.system") as mock_os_system:
        save_screenshot_path = "screenshot.png"
        adb_client.screenshot(save_screenshot_path)

        mock_os_system.assert_any_call(
            f"adb -s {adb_client.device_params.device_id}  shell screencap -p /sdcard/screenshot.png"
        )
        mock_os_system.assert_any_call(
            f"adb -s {adb_client.device_params.device_id}  pull /sdcard/screenshot.png {save_screenshot_path}"
        )


def test_tap(adb_client):
    with patch("os.system") as mock_os_system:
        coordinates = (100, 200)
        adb_client.tap(coordinates)

        mock_os_system.assert_called_once_with(
            f"adb -s {adb_client.device_params.device_id} shell input tap {coordinates[0]} {coordinates[1]}"
        )


def test_swipe(adb_client):
    with patch("os.system") as mock_os_system:
        start_coordinates = (100, 200)
        end_coordinates = (300, 400)
        swiping_time = 1000
        adb_client.swipe(start_coordinates, end_coordinates, swiping_time)

        mock_os_system.assert_called_once_with(
            f"adb -s {adb_client.device_params.device_id} shell input swipe {start_coordinates[0]} {start_coordinates[1]} {end_coordinates[0]} {end_coordinates[1]} {swiping_time}"
        )

if __name__ == "__main__":
    pytest.main(['pytest_metisse/test_adb.py'])



import cv2
from cv2 import Mat
from ..params import ImageRecognitionResult
import numpy as np  # type: ignore


def match_template(_screen_image_mat: Mat, _template_image_mat: Mat, _accuracy_val: float) -> ImageRecognitionResult:
    _img_recog_result = ImageRecognitionResult()
    image_x, image_y = _template_image_mat.shape[:2]
    result = cv2.matchTemplate(_screen_image_mat, _template_image_mat, cv2.TM_CCOEFF_NORMED)
    _, max_val, _, max_loc = cv2.minMaxLoc(result)  # type: ignore unused var

    if max_val > _accuracy_val:  # accuracy between two image
        _temp_center = (int(max_loc[0] + image_y / 2), int(max_loc[1] + image_x / 2))
        _img_recog_result.is_recognized = True
        _img_recog_result.coordinate = _temp_center
        _img_recog_result.recognition_threshold = max_val
        loc = np.where(result >= _accuracy_val)  # type: ignore
        loc_ren = len(loc[-1][:])  # type: ignore
        if loc_ren > 0:
            _temp_center_list = []
            for i in range(loc_ren):
                _temp_center = (int(loc[1][i] + image_y / 2), int(loc[0][i] + image_x / 2))  # type: ignore
                _temp_center_list.append(_temp_center)  # type: ignore
                #if (): print("pos:", _temp_center)
            _img_recog_result.coordinates_list = _temp_center_list
    else:
        _img_recog_result.is_recognized = False
        _img_recog_result.coordinate = (0, 0)
        _img_recog_result.recognition_threshold = max_val
    return _img_recog_result



# -*- coding=UTF-8 -*-
# pyright: strict
import os
from typing import Tuple
from ..client import Client
from ...params import DeviceParams


class AdbClient(Client):
    """
    android client
    """

    def __init__(self, device_params: DeviceParams):
        assert device_params.os_environment == 'android', 'device_params.os_environment must be android'
        self.device_params = device_params

    def screenshot(self, save_screenshot_path: str) -> None:
        """
        take a screenshot
        """
        os.system(f"adb -s {self.device_params.device_id}  shell screencap -p /sdcard/screenshot.png")
        os.system(f"adb -s {self.device_params.device_id}  pull /sdcard/screenshot.png {save_screenshot_path}")

    def tap(self, coordinates: Tuple[int, int]) -> None:
        """
        tap device
        """
        _x, _y = coordinates
        os.system(f"adb -s {self.device_params.device_id} shell input tap {_x} {_y}")

    def swipe(self, start_coordinates: Tuple[int, int], end_coordinates: Tuple[int, int], swiping_time: int) -> None:
        """
        swipe device
        """
        _x, _y = start_coordinates
        _x2, _y2 = end_coordinates
        os.system(f"adb -s {self.device_params.device_id} shell input swipe {_x} {_y} {_x2} {_y2} {swiping_time}")


# -*- coding=UTF-8 -*-
# pyright: strict
"""
This module contains functions for android and ios script.
It includes device port connect, screen shoot ,image similarity calculation, device control.
"""

from __future__ import annotations
import logging
import os
import time
try:
    import wda  # type: ignore
except ImportError:
    print('No module')
from typing import Tuple
# pylint: disable=no-member
from PIL import Image
import inspect
from .params import ImageRecognitionParams, SaveParams, DeviceParams, UiClientParams, ImageRecognitionResult
from .template_metis import TemplateMetisClass
from .clients.ios.wda import WdaClient
from .clients.android.adb import AdbClient
from .utils.metisse_path import DevPath
from .utils.opencv_utils import Opencv_utils
from .utils.ui_client import UiClient
from .utils.metisse_log import MetisseLogger
from .utils import image_recognition
from .settings import Settings as ST


class MetisseClass(TemplateMetisClass):

    def __init__(self, device_id: str, relatively_path: str, pyqt6_ui_label: UiClientParams, os_environment: str = 'android'):
        assert device_id != ''
        if not relatively_path:
            self._relatively_path = MetisseClass.get_current_path()
        else:
            self._relatively_path = relatively_path

        super(TemplateMetisClass, self).__init__()

        self._device_id = self.check_device_id_path_valid(device_id)
        self._dev_path = DevPath(self._relatively_path)
        self._script_path = self._dev_path.create_extended_script_path(self._device_id)
        self._logger = MetisseLogger("MetisClass_logger",
                                     log_level=logging.DEBUG,
                                     log_file=os.path.join(self._script_path.device_id_path, "log",
                                                           self.get_time() + self._device_id + ".log"))
        assert os_environment in ST.OS_ENVIRONMENT
        self._os_environment = os_environment  # android , ios
        self.ios_device_scale = 2  # init var

        if self._os_environment == 'ios':
            self._client = WdaClient(DeviceParams(device_id, os_environment))
        elif self._os_environment == 'android':
            self._client = AdbClient(DeviceParams(device_id, os_environment))
        else:
            raise ValueError('os_environment must be android or ios')
        self._img_recog_result = ImageRecognitionResult()

        self._ui_client = UiClient(pyqt6_ui_label)

        self.is_backup = False
        self.backup_time = time.strftime("%Y-%m-%d_%H_%M_%S", time.localtime())
        assert isinstance(self.backup_time, str)
        self.screenshot_wait_time_increase: float = 1
        self.is_check_gamelog: bool = False

    @staticmethod
    def get_current_path() -> str:
        """
        get current path
        """
        caller_frame = inspect.stack()[2]
        caller_file_path = caller_frame.filename
        _path = os.path.dirname(os.path.abspath(caller_file_path))
        return _path

    def execute_time_sleep(self, wait_time: float = 0):
        self._logger.info("execute_time_sleep : wait_time= %.2f", wait_time)
        self._ui_client.send_log_to_ui(f"execute_time_sleep method : \n wait_time= {wait_time:.2f}")
        time.sleep(wait_time)

    def check_device_id_path_valid(self,input:str) -> str:
        if ':' in input:
            return input.replace(':', '_')
        return input

    @property
    def get_device_id(self) -> str:
        if self._device_id:
            return ' -s ' + self._device_id
        return ''

    def check_gamelog(self, params: ImageRecognitionParams):
        params.template_image_name = 'log_button'
        try:
            _screen_image_path = self._script_path.get_screen_image_path(params)
            if not os.path.isfile(_screen_image_path):
                self.screenshot()
            _template_image_path = self._script_path.get_template_image_path(params)
            if not os.path.isfile(_template_image_path):
                raise FileNotFoundError(f"FileNotFoundError: {_template_image_path}")
            self.opencv_utils = Opencv_utils(_screen_image_path, _template_image_path)
            self._img_recog_result = image_recognition.match_template(self.opencv_utils.screen_image_mat,
                                                                      self.opencv_utils.template_image_mat, params.accuracy_val)
        except FileNotFoundError as error_msg:
            self._logger.info("FileNotFoundError: %s", error_msg)
        except Exception as error:
            self._logger.error("An unexpected error occurred: %s", error)

        if self._img_recog_result.is_recognized:
            self._logger.info("match_template method : template_name=%s prob=%.4f accuracy_val=%.4f %s",
                              params.template_image_name, self._img_recog_result.recognition_threshold, params.accuracy_val,
                              self._img_recog_result.is_recognized)
            self._ui_client.send_image_path_to_ui(_image_path=_template_image_path)
            self._ui_client.send_log_to_ui(
                f"match_template method : \n template_name={params.template_image_name}  \n prob={self._img_recog_result.recognition_threshold:.4f} \n accuracy_val={params.accuracy_val:.4f} \n {self._img_recog_result.is_recognized}"
            )
            if self.is_backup and params.is_backup:
                self.save_screenshot_compression(
                    SaveParams(
                        save_image_primary_dir='storage',
                        save_image_name=
                        f'{self.get_time()}_{params.template_image_name}_{self._img_recog_result.recognition_threshold:.2f}{self._img_recog_result.is_recognized}',
                        save_image_secondary_dir=self.backup_time,
                        is_refresh_screenshot=False))
            self.tap(self._img_recog_result.coordinate)

    def get_time(self) -> str:
        return time.strftime("%Y-%m-%d_%H_%M_%S_", time.localtime())

    def screenshot(
        self,
        save_screenshot_name: str = 'tmp0.png',
        save_screenshot_root_key: str = 'temp_image',
        save_screenshot_additional_root: str = '',
    ) -> None:

        if self._os_environment == 'android':

            self._client.screenshot(
                f"{self._script_path.get_image_path(save_screenshot_name,save_screenshot_root_key,save_screenshot_additional_root)}"
            )
        elif self._os_environment == 'ios':
            self._client.screenshot(
                f"{self._script_path.get_image_path(save_screenshot_name,save_screenshot_root_key,save_screenshot_additional_root)}"
            )
        else:
            raise RuntimeError("Unsupported OS environment")
        self._logger.debug("adb_screenshot method : process %s", save_screenshot_name)
        self._ui_client.send_log_to_ui(f"adb_screenshot method : \n process {save_screenshot_name:s}")

    def check_image_recognition(self, params: ImageRecognitionParams) -> bool:
        save_params = SaveParams(
            save_image_primary_dir='storage',
            save_image_name=
            f'{self.get_time()}_{params.template_image_name}_{self._img_recog_result.recognition_threshold:.2f}{self._img_recog_result.is_recognized}',
            save_image_secondary_dir=self.backup_time,
            is_refresh_screenshot=False)

        def _single_time() -> bool:
            if params.compare_times_counter > 1 and params.is_refresh_screenshot is False:
                self._logger.info('warring efficient lost')
            for _num in range(params.compare_times_counter):
                if self.is_check_gamelog:
                    self.check_gamelog(ImageRecognitionParams())
                if params.is_refresh_screenshot:
                    time.sleep(params.screenshot_wait_time + self.screenshot_wait_time_increase)
                    self.screenshot(params.screen_image_name, params.screen_image_primary_dir,
                                    params.screen_image_secondary_dir)

                _screen_image_path = self._script_path.get_screen_image_path(params)
                _template_image_path = self._script_path.get_template_image_path(params)
                self.opencv_utils = Opencv_utils(_screen_image_path, _template_image_path)
                self._img_recog_result = image_recognition.match_template(self.opencv_utils.screen_image_mat,
                                                                          self.opencv_utils.template_image_mat,
                                                                          params.accuracy_val)

                self._logger.info("image_recognition method : template_name=%s  prob=%.4f accuracy_val=%.4f %s",
                                  params.template_image_name, self._img_recog_result.recognition_threshold, params.accuracy_val,
                                  self._img_recog_result.is_recognized)
                self._ui_client.send_image_path_to_ui(_image_path=_template_image_path)
                self._ui_client.send_log_to_ui(
                    f"image_recognition method : \n template_name={params.template_image_name}  \n prob={self._img_recog_result.recognition_threshold:.4f} \n accuracy_val={params.accuracy_val:.4f} \n {self._img_recog_result.is_recognized}"
                )
                if self._img_recog_result.is_recognized:
                    if self.is_backup and params.is_backup:
                        self.save_screenshot_compression(save_params)
                    return True
            return False

        def _multiple_times() -> bool:
            _screen_image_name_list = [f'tmp{x}' for x in range(params.repeatedly_screenshot_times)]
            if self.is_check_gamelog:
                self.check_gamelog(ImageRecognitionParams())
            _template_image_path = self._script_path.get_template_image_path(params)
            time.sleep(params.screenshot_wait_time + self.screenshot_wait_time_increase)

            for _num in range(params.compare_times_counter):
                for _temp_screen_image_name in _screen_image_name_list:

                    self.screenshot(_temp_screen_image_name, params.screen_image_primary_dir, params.screen_image_secondary_dir)
                for _temp_screen_image_name in _screen_image_name_list:
                    _screen_image_path = self._script_path.get_screen_image_path(params)
                    params.screen_image_name = _temp_screen_image_name
                    self.opencv_utils = Opencv_utils(_screen_image_path, _template_image_path)
                    self._img_recog_result = image_recognition.match_template(self.opencv_utils.screen_image_mat,
                                                                              self.opencv_utils.template_image_mat,
                                                                              params.accuracy_val)
                    self._logger.info("match_template method : template_name=%s  prob=%.4f accuracy_val=%.4f %s",
                                      params.template_image_name, self._img_recog_result.recognition_threshold,
                                      params.accuracy_val, self._img_recog_result.is_recognized)
                    self._ui_client.send_image_path_to_ui(_image_path=_template_image_path)
                    self._ui_client.send_log_to_ui(
                        f"match_template method : \n template_name={params.template_image_name}  \n prob={self._img_recog_result.recognition_threshold:.4f} \n accuracy_val={params.accuracy_val:.4f} \n {self._img_recog_result.is_recognized}"
                    )
                    if self._img_recog_result.is_recognized:
                        if self.is_backup and params.is_backup:
                            self.save_screenshot_compression(save_params)
                        return True
            return False

        if params.repeatedly_screenshot_times <= 1:
            if _single_time():
                return True
        else:
            if _multiple_times():
                return True
        if self.is_backup and params.is_backup:
            self.save_screenshot_compression(save_params)
        return False

    def tap(
            self,
            center: Tuple[int, int],
            tap_execute_counter_times: int = 1,
            tap_execute_wait_time: float = 0,
            tap_offset: Tuple[int, int] = (0, 0),
    ) -> None:

        for _tap_time in range(tap_execute_counter_times):
            time.sleep(tap_execute_wait_time)
            (_x, _y) = center
            _x += tap_offset[0]
            _y += tap_offset[1]
            self._client.tap((_x, _y))
            self._logger.info("tap method : (x,y) = %s offset = %s", center, tap_offset)
            self._ui_client.send_log_to_ui(f"tap method : \n (x,y) = {center} \n offset = {tap_offset}")

    def swipe(
        self,
        center: Tuple[int, int],
        swipe_offset_position: Tuple[int, int],
        swiping_time: int = 300,
        swipe_execute_counter_times: int = 1,
        swipe_execute_wait_time: float = 0,
    ) -> None:

        for _swipe_time in range(swipe_execute_counter_times):
            time.sleep(swipe_execute_wait_time)
            (_x, _y) = center
            (_x2, _y2) = swipe_offset_position
            self._client.swipe((_x, _y), (_x2, _y2), swiping_time)
            self._logger.info("swipe method : (x,y) = %s (x2,y2) = %s swiping_time = %.2f", center, swipe_offset_position,
                              swiping_time)
            self._ui_client.send_log_to_ui(
                f"swipe method : \n (x,y) = {center} \n (x2,y2) = {swipe_offset_position} \n swiping_time = {swiping_time}")

    def press(
        self,
        center: Tuple[int, int],
        pressing_time: int = 400,
        press_execute_counter_times: int = 1,
        press_execute_wait_time: float = 0,
    ) -> None:

        for _swipe_time in range(press_execute_counter_times):
            try:
                time.sleep(press_execute_wait_time)
                self.swipe(center, center, pressing_time)
                self._logger.info("adb_press method : (x,y) = %s pressing_time = %d", center, pressing_time)
                self._ui_client.send_log_to_ui(f"adb_press method : \n (x,y) = {center} \n pressing_time = {pressing_time}")
            except OSError as error_msg:
                self._logger.exception("adb_press method :  %s", error_msg)
                self._ui_client.send_log_to_ui(f"adb_press method : \n {error_msg}")

    def default_tap(
            self,
            params: ImageRecognitionParams,
            tap_execute_wait_time: float = 0.1,
            tap_execute_counter_times: int = 1,
            tap_offset: Tuple[int, int] = (0, 0),
    ) -> bool:
        save_params = SaveParams(
            save_image_primary_dir='storage',
            save_image_name=
            f'{self.get_time()}_{params.template_image_name}_{self._img_recog_result.recognition_threshold:.2f}{self._img_recog_result.is_recognized}',
            save_image_secondary_dir=self.backup_time,
            is_refresh_screenshot=False)
        if self.check_image_recognition(params):
            self.tap(self._img_recog_result.coordinate, tap_execute_counter_times, tap_execute_wait_time, tap_offset)
            self._logger.info("default_tap method : template_name=%s  prob=%.4f %s", params.template_image_name,
                              self._img_recog_result.recognition_threshold, self._img_recog_result.is_recognized)
            self._ui_client.send_log_to_ui(
                f"default_tap method : \n template_name={params.template_image_name}  \n prob={self._img_recog_result.recognition_threshold:.4f} \n {self._img_recog_result.is_recognized}"
            )
            if self.is_backup and params.is_backup:
                self.save_screenshot_compression(save_params)
            return True

        self._logger.info("default_tap method : template_name=%s  prob=%.4f %s", params.template_image_name,
                          self._img_recog_result.recognition_threshold, self._img_recog_result.is_recognized)
        self._ui_client.send_log_to_ui(
            f"default_tap method : \n template_name={params.template_image_name}  \n prob={self._img_recog_result.recognition_threshold:.4f} \n {self._img_recog_result.is_recognized}"
        )
        if self.is_backup and params.is_backup:
            self.save_screenshot_compression(save_params)
        return False

    def default_swipe(
        self,
        params: ImageRecognitionParams,
        swipe_offset_position: Tuple[int, int] = (0, 0),
        swiping_time: int = 300,
        swipe_execute_counter_times: int = 1,
        swipe_execute_wait_time: float = 0,
    ) -> bool:
        save_params = SaveParams(
            save_image_primary_dir='storage',
            save_image_name=
            f'{self.get_time()}_{params.template_image_name}_{self._img_recog_result.recognition_threshold:.2f}{self._img_recog_result.is_recognized}',
            save_image_secondary_dir=self.backup_time,
            is_refresh_screenshot=False)
        if self.check_image_recognition(params):
            self.swipe(self._img_recog_result.coordinate, swipe_offset_position, swiping_time, swipe_execute_counter_times,
                       swipe_execute_wait_time)
            if self.is_backup and params.is_backup:
                self.save_screenshot_compression(save_params)
            return True

        if self.is_backup and params.is_backup:
            self.save_screenshot_compression(save_params)
        return False

    def default_press(
        self,
        params: ImageRecognitionParams,
        pressing_time: int = 300,
        press_execute_counter_times: int = 1,
        press_execute_wait_time: float = 0,
    ) -> bool:
        save_params = SaveParams(
            save_image_primary_dir='storage',
            save_image_name=
            f'{self.get_time()}_{params.template_image_name}_{self._img_recog_result.recognition_threshold:.2f}{self._img_recog_result.is_recognized}',
            save_image_secondary_dir=self.backup_time,
            is_refresh_screenshot=False)
        if self.check_image_recognition(params):
            self.press(self._img_recog_result.coordinate, pressing_time, press_execute_counter_times, press_execute_wait_time)
            if self.is_backup and params.is_backup:
                self.save_screenshot_compression(save_params)
            return True

        if self.is_backup and params.is_backup:
            self.save_screenshot_compression(save_params)
        return False

    def save_screenshot_compression(self, save_params: SaveParams) -> None:
        if save_params.is_refresh_screenshot:
            time.sleep(save_params.screenshot_wait_time)
            self.screenshot()
        _img = Image.open(self._script_path.get_load_image_path(save_params))
        if save_params.is_save_image_name_add_time:
            _save_png_image_name = self.get_time() + save_params.save_image_name + '.png'
        else:
            _save_png_image_name = save_params.save_image_name + '.png'
        save_params.save_image_name = _save_png_image_name
        _image_path = self._script_path.get_save_image_path(save_params)
        self._script_path.check_path(os.path.dirname(_image_path))

        if save_params.compression != 1:
            (_w, _h) = _img.size
            _w = int(_w * save_params.compression)
            _h = int(_h * save_params.compression)
            _new_resize_img = _img.resize((_w, _h))
            _new_resize_img.save(_image_path)
            if not self.is_backup:
                self._logger.info("save_screenshot_compression method : raw data w=%d, h=%d compression = %.2f name=%s ", _w,
                                  _h, save_params.compression, _save_png_image_name)
                self._ui_client.send_log_to_ui(
                    f"save_screenshot_compression method :\n raw data w={_w}, h={_h}\n compression = {save_params.compression}\n name={_save_png_image_name} "
                )
        else:
            (_w, _h) = _img.size
            _img.save(_image_path)
            if not self.is_backup:
                self._logger.info("save_screenshot_compression method : raw data w=%d, h=%d name=%s", _w, _h,
                                  _save_png_image_name)
                self._ui_client.send_log_to_ui(
                    f"save_screenshot_compression method : \n raw data w={_w}, h={_h} \n name={_save_png_image_name}")

    def crop_screenshot(
        self,
        coordinate1_tuple1: Tuple[int, int],
        coordinate2_tuple2: Tuple[int, int],
        save_params: SaveParams,
    ) -> None:

        if save_params.is_refresh_screenshot:
            time.sleep(save_params.screenshot_wait_time)
            self.screenshot()
        _img = Image.open(self._script_path.get_load_image_path(save_params))
        _pos_x, _pos_y = coordinate1_tuple1
        _pos_x2, _pos_y2 = coordinate2_tuple2

        _pos_x2 -= _pos_x
        _pos_y2 -= _pos_y
        _region = (_pos_x, _pos_y, _pos_x + _pos_x2, _pos_y + _pos_y2)
        _crop_img = _img.crop(_region)
        if save_params.is_save_image_name_add_time:
            _save_png_image_name = self.get_time() + save_params.save_image_name + '.png'
        else:
            _save_png_image_name = save_params.save_image_name + '.png'

        save_params.save_image_name = _save_png_image_name
        _image_path = self._script_path.get_save_image_path(save_params)
        self._script_path.check_path(os.path.dirname(_image_path))
        _crop_img.save(_image_path)
        self._logger.info("crop_screenshot method : exported : w=%s", _save_png_image_name)
        self._ui_client.send_log_to_ui(f"crop_screenshot method : \n exported : w={_save_png_image_name}")



# -*- coding=UTF-8 -*-
# pyright: strict
from typing import Tuple
import wda  # type: ignore
from ..client import Client
from ...params import DeviceParams


class WdaClient(Client):
    """
    ios client
    """

    def __init__(self, device_params: DeviceParams, skip_wda_launch: bool = False):
        assert device_params.os_environment == 'ios', 'device_params.os_environment must be ios'
        self.device_params = device_params
        if not skip_wda_launch:
            self.wda_client = wda.USBClient(self.device_params.device_id, port=8100)
        self.ios_device_scale = 2
        try:
            self.ios_device_scale = self.wda_client.scale  # type: ignore
        except AttributeError:
            self.ios_device_scale = 2

    def screenshot(self, save_screenshot_path: str) -> None:
        """
        take a screenshot
        """
        self.wda_client.screenshot().save(save_screenshot_path)  # type: ignore

    def tap(self, coordinates: Tuple[int, int]) -> None:
        """
        tap device
        """
        _x, _y = coordinates
        self.wda_client.tap(int(_x / self.ios_device_scale), int(_y / self.ios_device_scale))  # type: ignore

    def swipe(self, start_coordinates: Tuple[int, int], end_coordinates: Tuple[int, int], swiping_time: int) -> None:
        """
        swipe device
        """
        _x, _y = start_coordinates
        _x2, _y2 = end_coordinates
        _ios_swipe_time = float(swiping_time) / 1000
        self.wda_client.swipe(  # type: ignore
            int(_x / self.ios_device_scale),  # type: ignore
            int(_y / self.ios_device_scale),  # type: ignore
            int(_x2 / self.ios_device_scale),  # type: ignore
            int(_y2 / self.ios_device_scale),  # type: ignore
            _ios_swipe_time)  # type: ignore



import os
import sys
from  unittest import mock
import numpy as np
import pytest
from PyQt6.QtWidgets import QApplication, QLabel
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.params import UiClientParams
from metisse.utils.ui_client import UiClient

app = QApplication(sys.argv)

@pytest.fixture
def ui_client_setup():
    log_label = QLabel()
    image_label = QLabel()
    ui_client_params = UiClientParams(log_label, image_label)
    ui_client = UiClient(ui_client_params)
    yield ui_client, ui_client_params
    del ui_client

def test_send_log_to_ui(ui_client_setup):
    ui_client, ui_client_params = ui_client_setup
    test_log = "Test log message"
    ui_client.send_log_to_ui(test_log)
    assert ui_client_params.log_label.text() == test_log

def test_send_image_path_to_ui(ui_client_setup):
    ui_client, ui_client_params = ui_client_setup
    with mock.patch("cv2.imread") as mock_imread:
        mock_image = np.zeros((100, 100, 3), dtype=np.uint8)
        mock_imread.return_value = mock_image

        test_image_path = "test_image_path.jpg"
        ui_client.send_image_path_to_ui(test_image_path)

        mock_imread.assert_called_once_with(test_image_path)
        assert ui_client_params.image_label.pixmap() is not None
if __name__ == '__main__':
    pytest.main(['-v','-s','pytest_metisse/test_ui_client.py'])


# Form implementation generated from reading ui file 'c:\Users\henrychen\Documents\gitlab\metisse\metisse\example\ui\universal_script_gui_main.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(458, 623)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.script_listWidget = QtWidgets.QListWidget(self.centralwidget)
        self.script_listWidget.setGeometry(QtCore.QRect(50, 140, 181, 281))
        self.script_listWidget.setObjectName("script_listWidget")
        self.lang_listWidget = QtWidgets.QListWidget(self.centralwidget)
        self.lang_listWidget.setGeometry(QtCore.QRect(230, 140, 51, 281))
        self.lang_listWidget.setObjectName("lang_listWidget")
        self.device_listWidget = QtWidgets.QListWidget(self.centralwidget)
        self.device_listWidget.setGeometry(QtCore.QRect(280, 140, 131, 281))
        self.device_listWidget.setObjectName("device_listWidget")
        self.excute_button = QtWidgets.QPushButton(self.centralwidget)
        self.excute_button.setGeometry(QtCore.QRect(50, 420, 231, 81))
        self.excute_button.setObjectName("excute_button")
        self.renew_button = QtWidgets.QPushButton(self.centralwidget)
        self.renew_button.setGeometry(QtCore.QRect(280, 420, 131, 81))
        self.renew_button.setObjectName("renew_button")
        self.text_label_show_info = QtWidgets.QLabel(self.centralwidget)
        self.text_label_show_info.setGeometry(QtCore.QRect(50, 10, 361, 101))
        self.text_label_show_info.setObjectName("text_label_show_info")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setGeometry(QtCore.QRect(110, 120, 51, 20))
        self.label_2.setObjectName("label_2")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setGeometry(QtCore.QRect(240, 120, 31, 20))
        self.label_3.setObjectName("label_3")
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setGeometry(QtCore.QRect(330, 120, 31, 20))
        self.label_4.setObjectName("label_4")
        self.sub2_button = QtWidgets.QPushButton(self.centralwidget)
        self.sub2_button.setGeometry(QtCore.QRect(280, 500, 131, 81))
        self.sub2_button.setObjectName("sub2_button")
        self.sub3_button = QtWidgets.QPushButton(self.centralwidget)
        self.sub3_button.setGeometry(QtCore.QRect(50, 500, 231, 81))
        self.sub3_button.setObjectName("sub3_button")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 458, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.excute_button.setText(_translate("MainWindow", "執行選擇腳本"))
        self.renew_button.setText(_translate("MainWindow", "刷新裝置清單"))
        self.text_label_show_info.setText(_translate("MainWindow", "test"))
        self.label_2.setText(_translate("MainWindow", "選擇腳本"))
        self.label_3.setText(_translate("MainWindow", "語系"))
        self.label_4.setText(_translate("MainWindow", "裝置"))
        self.sub2_button.setText(_translate("MainWindow", "調整android裝置"))
        self.sub3_button.setText(_translate("MainWindow", "選擇腳本說明(起始位置)"))



# Form implementation generated from reading ui file 'c:\Users\henrychen\Documents\gitlab\metisse\metisse\example\ui\universal_script_gui_sub.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):

    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(456, 537)
        self.stop_Button = QtWidgets.QPushButton(Dialog)
        self.stop_Button.setGeometry(QtCore.QRect(170, 460, 101, 51))
        self.stop_Button.setObjectName("stop_Button")
        self.record_label = QtWidgets.QLabel(Dialog)
        self.record_label.setGeometry(QtCore.QRect(30, 20, 221, 61))
        self.record_label.setObjectName("record_label")
        self.lcdNumber = QtWidgets.QLCDNumber(Dialog)
        self.lcdNumber.setGeometry(QtCore.QRect(260, 30, 101, 41))
        self.lcdNumber.setDigitCount(6)
        self.lcdNumber.setObjectName("lcdNumber")
        self.log_label = QtWidgets.QLabel(Dialog)
        self.log_label.setGeometry(QtCore.QRect(70, 190, 321, 121))
        self.log_label.setObjectName("log_label")
        self.image_label = QtWidgets.QLabel(Dialog)
        self.image_label.setGeometry(QtCore.QRect(70, 310, 321, 141))
        self.image_label.setObjectName("image_label")
        self.info_label = QtWidgets.QLabel(Dialog)
        self.info_label.setGeometry(QtCore.QRect(70, 100, 301, 81))
        self.info_label.setObjectName("info_label")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.stop_Button.setText(_translate("Dialog", "中止"))
        self.record_label.setText(
            _translate("Dialog", "<html><head/><body><p><span style=\" font-size:36pt;\">執行時間 :</span></p></body></html>"))
        self.log_label.setText(_translate("Dialog", "TextLabel"))
        self.image_label.setText(_translate("Dialog", "TextLabel"))
        self.info_label.setText(_translate("Dialog", "TextLabel"))



class Settings(object):
    DEV_ENVIRONMENT_ROOT_PATH = ['icon', 'ui']  #,'script'
    SCRIPT_ENVIRONMENT_ROOT_PATH = ['temp_image', 'storage', 'backup', 'log']
    OS_ENVIRONMENT = ['android', 'ios']



# -*- coding=UTF-8 -*-
# pyright: strict
import importlib
import os
import sys
import time
from dataclasses import dataclass
import cv2
import ctypes
from typing import Any, Dict, List
from multiprocessing import Pool
import threading
import dill  # type: ignore
from PyQt6 import QtWidgets, uic
from PyQt6.QtCore import QTimer, QDateTime
from PyQt6.QtWidgets import QMainWindow
from PyQt6.QtGui import QImage, QPixmap
from gui_settings import UiSettings as UI_ST

curPath = os.path.abspath(os.path.dirname(__file__))

@dataclass
class UiClientParams:
    image_label: QtWidgets.QLabel = None  # type: ignore
    log_label: QtWidgets.QLabel = None  # type: ignore

class MainWindow(QMainWindow):

    def __init__(self, *args: Any, **kwargs: Any):


        self.lang_list = ['_tc', '_sc', '_jp', '_en', '_kr']
        directory = os.path.dirname(curPath)  #upper level directory
        self.loaded_scripts = self.load_scripts_from_directory(directory)
        self.script_list = []
        # dynamic import script
        for script_name, _ in self.loaded_scripts.items():
            print(f"load script: {script_name}")
            self.script_list.append(script_name)

        super(MainWindow, self).__init__(*args, **kwargs)
        self.setup_ui()
        self.current_select_device_name: str
        self.current_select_device_id: str
        self.current_select_device_envi: str
        self.current_select_lang: str
        self.current_select_script: str
        self.sub_obj_dict = {str: Dialog}

    def load_scripts_from_directory(self, directory: str, script_prefix: str = "script"):
        scripts = {}

        for file in os.listdir(directory):
            if file.startswith(script_prefix) and file.endswith(".py"):
                script_name = file[:-3]
                script_path = os.path.join(directory, file)

                spec = importlib.util.spec_from_file_location(script_name, script_path)
                script_module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(script_module)

                scripts[script_name] = script_module

        return scripts

    def setup_ui(self):
        #Load the UI Page by PyQt6
        uic.loadUi(os.path.join(curPath, UI_ST.LOAD_MAIN_UI), self)  # type: ignore
        self.setWindowTitle(UI_ST.LOAD_MAIN_UI)
        self.set_listWidget()

        # signal
        self.script_listWidget.currentItemChanged.connect(self.set_info)
        self.lang_listWidget.currentItemChanged.connect(self.set_info)
        self.device_listWidget.currentItemChanged.connect(self.set_info)

        self.excute_button.clicked.connect(self.sub_ui)
        self.renew_button.clicked.connect(self.renew)
        self.sub2_button.clicked.connect(self.sub2_ui)
        self.sub3_button.clicked.connect(self.sub3_ui)

    def set_listWidget(self):
        self.devices_dict: dict[str, list[str]] = {}
        _list_devices: Dict[str, List[str]] = {}
        _list_devices.update(self.readDevicesList_ios())
        _list_devices.update(self.readDevicesList_andriod())
        for _i_key, _i_value in _list_devices.items():
            _RealDeviceName = _i_key
            self.devices_dict[_RealDeviceName] = _i_value
            self.device_listWidget.addItem(_RealDeviceName)
        for _i in range(len(self.lang_list)):
            self.lang_listWidget.addItem(self.lang_list[_i])
        for _i in range(len(self.script_list)):
            self.script_listWidget.addItem(self.script_list[_i])

    def set_info(self):
        try:
            self.script_listWidget.currentItem().text()
            self.current_select_script = self.script_listWidget.currentItem().text()
        except Exception as _e:
            print(_e)
            self.current_select_script = ''
        try:
            self.lang_listWidget.currentItem().text()
            self.current_select_lang = self.lang_listWidget.currentItem().text()
        except Exception as _e:
            print(_e)
            self.current_select_lang = '_tw'
        try:

            self.current_select_device_name = self.device_listWidget.currentItem().text()
            self.current_select_device_id = self.devices_dict[self.current_select_device_name][0]
            self.current_select_device_envi = self.devices_dict[self.current_select_device_name][1]
        except Exception as _e:
            print(_e)
            self.current_select_device_name = ''
            self.current_select_device_id = ''
            self.current_select_device_envi = ''
        self.text_label_show_info.setText('目前腳本: {}   語言: {} \n目前裝置: {} \n裝置 sid : {} \n裝置環境: {}'.format(
            self.current_select_script,
            self.current_select_lang,
            self.current_select_device_name,
            self.current_select_device_id,
            self.current_select_device_envi,
        ))

    def readDevicesList_ios(self) -> Dict[str, List[str]]:
        devicesNames_dict: Dict[str, List[str]] = {}
        try:
            p = os.popen('tidevice list')
            devicesList = p.read()
            p.close()
            if devicesList == '': raise Exception('please check tidevice !')
            info_lists = devicesList.split("\n")
            for info_row_str in info_lists:
                info_row_list = list(filter(None, info_row_str.split('  ')))
                if info_row_list and info_row_list[0] != 'UDID':
                    if info_row_list[2] in devicesNames_dict:
                        print('ios device name existed !')
                    devicesNames_dict[info_row_list[2]] = [info_row_list[0], 'ios']
        except Exception as e:
            print(e)
        return devicesNames_dict

    def readDevicesList_andriod(self) -> Dict[str, List[str]]:
        _p = os.popen('adb devices')
        _devices_list = _p.read()
        _p.close()
        _lists = _devices_list.split("\n")
        _devices_name_list: Dict[str, List[str]] = {}

        for _item in _lists:
            if (_item.strip() == ""):
                continue
            elif (_item.startswith("List of")):
                continue
            else:
                _devices_name_list[self.getRealDeviceName_android(_item.split("\t")[0])] = [_item.split("\t")[0], 'android']

        return _devices_name_list

    def getRealDeviceName_android(self, _deviceid: str):
        p = os.popen('adb -s ' + _deviceid + ' shell getprop ro.product.manufacturer')
        manufacturer = p.read()
        p.close()
        p = os.popen('adb -s ' + _deviceid + ' shell getprop ro.product.model')
        model = p.read()
        p.close()
        return manufacturer.strip() + " " + model.strip()

    # slot
    def sub_ui(self) -> None:

        self.set_info()

        pk = dill.dumps(self.create_small_window())  # type: ignore
        new_obj = dill.loads(pk)  # type: ignore
        pool = Pool(processes=2)
        pool_outputs = pool.map(new_obj, [])  # type: ignore
        pool.close()
        #pool.join() #debug
        time.sleep(1)

    def create_small_window(self):

        self.sub_obj_dict[self.current_select_device_name] = Dialog(  # type: ignore
            select_device_name=self.current_select_device_name,
            select_device_id=self.current_select_device_id,
            select_lang=self.current_select_lang,
            select_script=self.current_select_script,
            select_device_envi=self.current_select_device_envi,
            script_list=self.script_list,
            loaded_scripts=self.loaded_scripts,
        )  # type: ignore
        self.sub_obj_dict[self.current_select_device_name].thread_t1()  # type: ignore

    def get_select_device(self) -> str:
        try:
            _select_currentItem = self.device_listWidget.currentItem().text()
            self.text_label_show_info.setText('目前選擇裝置: ' + _select_currentItem)
            return _select_currentItem
        except Exception as _e:
            print(_e)
            self.text_label_show_info.setText('未選擇裝置')
            return ''

    def renew(self) -> None:
        self.device_listWidget.clear()
        self.lang_listWidget.clear()
        self.script_listWidget.clear()
        self.set_listWidget()

    def sub2_ui(self) -> None:
        self.set_info()
        self.sub2 = Dialog2(
            select_device_name=self.current_select_device_name,
            select_device_id=self.current_select_device_id,
            select_device_envi=self.current_select_device_envi,
        )

    def sub3_ui(self) -> None:
        self.sub3 = Dialog3(select_script=self.current_select_script)


class Dialog(QtWidgets.QDialog):

    def __init__(self,
                 select_device_name: str = '',
                 select_device_id: str = '',
                 select_lang: str = '',
                 select_script: str = '',
                 select_device_envi: str = '',
                 script_list: List[str] = [],
                 loaded_scripts=None,
                 *args: Any,
                 **kwargs: Any):
        super(Dialog, self).__init__(*args, **kwargs)
        uic.loadUi(os.path.join(curPath, UI_ST.LOAD_SUB_UI[0]), self)  # type: ignore
        self.setWindowTitle(UI_ST.LOAD_SUB_UI[0])
        # signal
        self.script_list = script_list
        self.loaded_scripts = loaded_scripts
        self.stop_Button.clicked.connect(self.stop)
        self.select_device_name = select_device_name
        self.select_device_id = select_device_id
        self.select_lang = select_lang
        self.select_script = select_script
        self.select_device_envi = select_device_envi
        self.info_label.setText('目前腳本: {}   語言: {} \n目前裝置: {} \n裝置 sid : {} \n裝置環境: {}'.format(
            self.select_script,
            self.select_lang,
            self.select_device_name,
            self.select_device_id,
            self.select_device_envi,
        ))
        self.flag_end = True
        self.log_label.setText('')
        self.image_label.setText('')

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.refresh)  # type: ignore
        self.startDate = QDateTime.currentMSecsSinceEpoch()
        self.timer.start()

        self.t1: threading.Thread
        self.show()

    # slot
    def stop(self) -> None:
        self.close()
        ctypes.pythonapi.PyThreadState_SetAsyncExc(self.t1.native_id, ctypes.py_object(SystemExit))

    def closeEvent(self, a0):
        self.close()
        ctypes.pythonapi.PyThreadState_SetAsyncExc(self.t1.native_id, ctypes.py_object(SystemExit))

    def set_text(self):
        self.record_label.setText('錄影剩餘時間')
        self.startDate = QDateTime.currentMSecsSinceEpoch()

    def refresh(self):
        if self.flag_end:
            self.nowDate = QDateTime.currentMSecsSinceEpoch()
            interval = self.nowDate - self.startDate
            if interval > 0:
                days = interval // (24 * 60 * 60 * 1000)
                hour = (interval - days * 24 * 60 * 60 * 1000) // (60 * 60 * 1000)
                min = (interval - days * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000) // (60 * 1000)
                sec = (interval - days * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000 - min * 60 * 1000) // 1000
                intervals = str(min) + ':' + str(sec)
                self.lcdNumber.display(intervals)
        else:
            self.destroy()

    def thread_t1(self):
        self.t1 = threading.Thread(
            name=self.select_device_id,
            target=self.execute_select_script,  # type: ignore
            args=[])  #build thread

        self.t1.daemon = True
        self.t1.start()  #執行

    def execute_select_script(self):
        # com.pinkcore.heros
        _temp_args = {
            'device_id': self.select_device_id,
            'relatively_path': None,
            'pyqt6_ui_label': UiClientParams(image_label=self.image_label , log_label=self.log_label),
            'os_environment': self.select_device_envi,
        }

        script_module = self.loaded_scripts[self.select_script]
        script_instance = getattr(script_module, self.select_script)(**_temp_args)
        script_instance(self.select_lang)

        self.flag_end = False


class Dialog2(QtWidgets.QDialog):

    def __init__(self,
                 select_device_name: str = '',
                 select_device_id: str = '',
                 select_device_envi: str = '',
                 *args: Any,
                 **kwargs: Any):
        super(Dialog2, self).__init__(*args, **kwargs)

        self.select_device_name = select_device_name
        self.select_device_id = select_device_id
        self.select_device_envi = select_device_envi

        self.setup_ui()
        self.show()

    def setup_ui(self):
        uic.loadUi(os.path.join(curPath, UI_ST.LOAD_SUB_UI[1]), self)  # type: ignore
        self.setWindowTitle(UI_ST.LOAD_SUB_UI[1])
        #153
        list_screen_size = UI_ST.DEVICE_SCREEN_SIZE
        list_screen_dpi = UI_ST.DEVICE_SCREEN_DENSITY
        for screen_size in list_screen_size:
            self.size_listWidget.addItem(screen_size)
        for screen_dpi in list_screen_dpi:
            self.dpi_listWidget.addItem(screen_dpi)
        self.screen_size = '1280x720'
        self.screen_dpi = '160'
        # slot

        self.info_label.setText('目前裝置: {} \n裝置 sid : {} \n裝置環境: {}\n解析度: {}dpi: {}'.format(
            self.select_device_name,
            self.select_device_id,
            self.select_device_envi,
            self.get_device_size(),
            self.get_device_dpi(),
        ))
        self.size_listWidget.currentItemChanged.connect(self.set_screen_size)
        self.dpi_listWidget.currentItemChanged.connect(self.set_screen_dpi)
        self.stop_Button.clicked.connect(self.stop)
        self.adjust_button.clicked.connect(
            lambda: os.system("adb -s {} shell wm size {}".format(self.select_device_id, self.screen_size)))
        self.reset_button.clicked.connect(lambda: os.system("adb -s {} shell wm size reset".format(self.select_device_id)))
        self.dpi_adjust_button.clicked.connect(
            lambda: os.system("adb -s {} shell wm density {}".format(self.select_device_id, self.screen_dpi)))
        self.dpi_reset_button.clicked.connect(
            lambda: os.system("adb -s {} shell wm density reset".format(self.select_device_id)))
        self.adb_button.clicked.connect(lambda: os.system("adb devices"))
        self.adb_kill_button.clicked.connect(lambda: os.system("adb kill-server"))
        self.get_device_info_Button.clicked.connect(self.get_device_info)

    def set_screen_size(self):
        self.screen_size = self.size_listWidget.currentItem().text()

    def set_screen_dpi(self):
        self.screen_dpi = self.dpi_listWidget.currentItem().text()

    def get_device_info(self):
        self.info_label.setText('目前裝置: {} \n裝置 sid : {} \n裝置環境: {}\n解析度: {}dpi: {}'.format(
            self.select_device_name,
            self.select_device_id,
            self.select_device_envi,
            self.get_device_size(),
            self.get_device_dpi(),
        ))

    def get_device_size(self):
        return os.popen('adb -s {}  shell wm size'.format(self.select_device_id)).read()

    def get_device_dpi(self):
        return os.popen('adb -s {}  shell wm density'.format(self.select_device_id)).read()

    def stop(self) -> None:
        self.close()


class Dialog3(QtWidgets.QDialog):

    def __init__(self, select_script: str = '', *args: Any, **kwargs: Any):
        super(Dialog3, self).__init__(*args, **kwargs)

        self.select_script = select_script

        self.setup_ui()
        self.show()

    def setup_ui(self):
        uic.loadUi(os.path.join(curPath, UI_ST.LOAD_SUB_UI[2]), self)  # type: ignore
        self.setWindowTitle(UI_ST.LOAD_SUB_UI[2])
        _image_path = curPath + f'\\script_info\\{self.select_script}.png'
        self._pyqt_img = cv2.imread(_image_path)

        height, width, _ = self._pyqt_img.shape

        bytesPerline = 3 * width
        self.qimg = QImage(self._pyqt_img, width, height, bytesPerline, QImage.Format.Format_RGB888).rgbSwapped()  #type: ignore
        self.qimg = self.qimg.scaled(1266, 585)
        self.img_label.setPixmap(QPixmap.fromImage(self.qimg))


def main():
    app = QtWidgets.QApplication(sys.argv)
    main = MainWindow()
    main.show()

    sys.exit(app.exec())


if __name__ == '__main__':
    main()

#  pyinstaller -D -p #'curPath'


# -*- coding=UTF-8 -*-
# pyright: strict
from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Tuple

from .params import ImageRecognitionParams, SaveParams


class TemplateMetisClass(ABC):
    """_summary_
        adb fuction template
    Args:
        ABC (_type_): Abstract Base Classes
    """

    @property
    @abstractmethod
    def get_device_id(self) -> str:
        ...

    @abstractmethod
    def check_image_recognition(self, params: ImageRecognitionParams) -> bool:
        ...

    @abstractmethod
    def screenshot(
        self,
        save_screenshot_name: str,
        save_screenshot_root_key: str,
        save_screenshot_additional_root: str,
    ) -> None:
        ...

    @abstractmethod
    def tap(self, center: Tuple[int, int], tap_execute_counter_times: int, tap_execute_wait_time: float,
            tap_offset: Tuple[int, int]) -> None:
        ...

    @abstractmethod
    def default_tap(
        self,
        params: ImageRecognitionParams,
        tap_execute_wait_time: float,
        tap_execute_counter_times: int,
        tap_offset: Tuple[int, int],
    ) -> bool:
        ...

    @abstractmethod
    def swipe(
        self,
        center: Tuple[int, int],
        swipe_offset_position: Tuple[int, int],
        swiping_time: int,
        swipe_execute_counter_times: int,
        swipe_execute_wait_time: float,
    ) -> None:
        ...

    @abstractmethod
    def press(
        self,
        center: Tuple[int, int],
        pressing_time: int,
        press_execute_counter_times: int,
        press_execute_wait_time: float,
    ) -> None:
        ...

    @abstractmethod
    def save_screenshot_compression(self, save_params: SaveParams) -> None:
        ...

    @abstractmethod
    def crop_screenshot(
        self,
        coordinate1_tuple1: Tuple[int, int],
        coordinate2_tuple2: Tuple[int, int],
        save_params: SaveParams,
    ) -> None:
        ...




from ..params import UiClientParams
from PyQt6.QtGui import QImage, QPixmap
import cv2


class UiClient(object):

    def __init__(self, ui_client_params: UiClientParams) -> None:
        self.ui_client_params = ui_client_params

    def send_log_to_ui(self, _log_message: str):
        if self.ui_client_params and self.ui_client_params.log_label:
            self.ui_client_params.log_label.setText(_log_message)

    def send_image_path_to_ui(self, _image_path: str):
        if self.ui_client_params and self.ui_client_params.image_label:
            self._pyqt_img = cv2.imread(_image_path)
            _height, _width, _ = self._pyqt_img.shape
            _bytes_perline = 3 * _width
            self.qimg = QImage(
                self._pyqt_img,  # type: ignore
                _width,
                _height,
                _bytes_perline,  # type: ignore
                QImage.Format.Format_RGB888).rgbSwapped()  # type: ignore
            self.ui_client_params.image_label.setPixmap(QPixmap.fromImage(self.qimg))


import os
import sys
import pytest
from unittest import mock
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.settings import Settings as ST
from metisse.utils.metisse_path import DevPath


@pytest.fixture
def test_dev_path_setup():
    with mock.patch("builtins.print") as print_mock:
        relative_path = "pytest_metisse"
        absolute_path = os.path.abspath(relative_path)

        yield absolute_path, print_mock

        for _document_path in ST.DEV_ENVIRONMENT_ROOT_PATH:
            if os.path.exists(os.path.join(absolute_path, _document_path)):
                os.removedirs(os.path.join(absolute_path, _document_path))


def test_init(test_dev_path_setup):
    absolute_path, print_mock = test_dev_path_setup
    dev_path = DevPath(absolute_path)
    assert dev_path._absolute_path == absolute_path



def test_initialize_dev_environment(test_dev_path_setup):
    absolute_path, _ = test_dev_path_setup
    dev_path = DevPath(absolute_path)
    dev_path.initialize_dev_environment()

    for _document_path in ST.DEV_ENVIRONMENT_ROOT_PATH:
        _document_path_temp = os.path.join(absolute_path, _document_path)
        assert os.path.isdir(_document_path_temp)


def test_auto_generate_dev_path(test_dev_path_setup):
    absolute_path, _ = test_dev_path_setup
    DevPath.auto_generate_dev_path()

    for _document_path in ST.DEV_ENVIRONMENT_ROOT_PATH:
        _document_path_temp = os.path.join(absolute_path, _document_path)
        print(f"Checking directory: {_document_path_temp}")  # Add this line to print debug info
        assert os.path.isdir(_document_path_temp)
if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_metis_path.py'])


# -*- coding=UTF-8 -*-
# pyright: strict
from dataclasses import dataclass, field
from PyQt6 import QtWidgets
from typing import List, Tuple


@dataclass
class ImageRecognitionParams:
    template_image_name: str = ''
    compare_times_counter: int = 1
    screenshot_wait_time: float = 0.1
    accuracy_val: float = 0.9
    is_refresh_screenshot: bool = True
    screen_image_name: str = 'tmp0'
    screen_image_primary_dir: str = 'temp_image'
    screen_image_secondary_dir: str = ''
    screen_image_subdirs: List[str] = field(default_factory=list)
    template_image_primary_dir: str = 'icon'
    template_image_secondary_dir: str = ''
    template_image_subdirs: List[str] = field(default_factory=list)
    is_backup: bool = True
    repeatedly_screenshot_times: int = 1


@dataclass
class SaveParams:
    load_image_primary_dir: str = 'temp_image'
    save_image_primary_dir: str = 'storage'
    save_image_name: str = ''
    screenshot_wait_time: float = 0.1
    compression: float = 1
    load_image_name: str = 'tmp0.png'
    save_image_secondary_dir: str = ''
    save_image_subdirs: List[str] = field(default_factory=list)
    load_image_secondary_dir: str = ''
    load_image_subdirs: List[str] = field(default_factory=list)

    is_save_image_name_add_time: bool = False
    is_refresh_screenshot: bool = True


@dataclass
class DeviceParams:
    device_id: str = ''
    os_environment: str = ''


@dataclass
class UiClientParams:
    image_label: QtWidgets.QLabel = None  # type: ignore
    log_label: QtWidgets.QLabel = None  # type: ignore


@dataclass
class ImageRecognitionResult:
    is_recognized: bool = False
    coordinate: Tuple[int, int] = (0, 0)
    coordinates_list: List[Tuple[int, int]] = field(default_factory=list)
    recognition_threshold: float = 0.9


from setuptools import setup, find_packages
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
setup(
    name='metisse',
    version="0.0.7",
    description="A versatile and automated testing framework for games and apps on Android and iOS platforms",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Henry Chen',
    author_email='weekand7@gmail.com',
    license='Apache License 2.0',
    keywords=['automation', 'automated-test', 'game', 'android', 'ios'],
    packages=find_packages(exclude=['unit_tests', 'unit_tests.*']),
    package_data={
        'metisse': ['example/example_data/**/*'],
    },
    install_requires=[
        'facebook_wda>=1.3.3',
        'numpy>=1.17.3',
        'opencv_python>=4.5.1.48',
        'Pillow>=3.4.0',
        'PyQt6>=6.0.0',
        'dill>=0.3.1.1',
        'tidevice>=0.9.12',
    ],
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



import shutil
import tempfile
import os
import sys
import pytest
from PIL import Image


sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from metisse.params import SaveParams, UiClientParams
from metisse.metisse import MetisseClass

curPath = os.path.abspath(os.path.dirname(__file__))



@pytest.fixture
def test_test_save_screenshot_compression_setup():
    test_dir = tempfile.mkdtemp()
    curPath = os.path.abspath(os.path.dirname(__file__))
    test_metisse = MetisseClass(
        device_id='test_virtual_device',
        relatively_path=test_dir,
        pyqt6_ui_label=UiClientParams(),
        os_environment='android',
    )
    test_metisse._logger.close()
    test_metisse.is_backup = False
    test_metisse.screenshot_wait_time_increase = 1
    test_metisse.is_check_gamelog = False

    src_test_images = os.path.join(curPath,'test_data','image','tmp0.png')
    dst_test_images = os.path.join(test_dir,'test_virtual_device','temp_image' ,'tmp0.png')
    shutil.copy(src_test_images, dst_test_images)

    yield test_metisse
    shutil.rmtree(test_dir)

def test_save_screenshot_compression(test_test_save_screenshot_compression_setup):
    test_metisse = test_test_save_screenshot_compression_setup
    save_params = SaveParams(load_image_primary_dir='temp_image',
                                load_image_name='tmp0',
                                save_image_primary_dir='storage',
                                save_image_name='save0',
                                compression=0.5,
                                is_refresh_screenshot=False,
                                is_save_image_name_add_time=False)

    test_metisse.save_screenshot_compression(save_params)

    expected_output_path = os.path.join(test_metisse._script_path.device_id_path, save_params.save_image_primary_dir,
                                        save_params.save_image_name)
    print(expected_output_path)
    assert os.path.exists(expected_output_path) , 'Output file not found.'
    expected_output_path = os.path.join(test_metisse._script_path.device_id_path, save_params.save_image_primary_dir,
                                        save_params.save_image_name)
    assert os.path.exists(expected_output_path) , 'Output file not found.'

    with Image.open(expected_output_path) as output_image:  # avoid ResourceWarning
        with Image.open(
                os.path.join(test_metisse._script_path.device_id_path, save_params.load_image_primary_dir,
                                test_metisse._script_path._check_image_name_pngFormat(
                                    save_params.load_image_name))) as original_image:

            expected_size = tuple(int(x * save_params.compression) for x in original_image.size)
            assert output_image.size == expected_size, 'Output image size does not match expected size.'


if __name__ == '__main__':
    pytest.main(['-v','-s','pytest_metisse/test_save_screenshot_compression.py'])



import os
import sys
import pytest
import cv2
from cv2 import Mat
import numpy as np
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
# Import other necessary modules and classes
curPath = os.path.abspath(os.path.dirname(__file__))
from metisse.utils.image_recognition import match_template
from metisse.params import ImageRecognitionResult


@pytest.fixture()
def test_match_setup():
    # Load test images
    screen_image_path = os.path.join(curPath,'test_data','image', 'tmp0.png')
    template_image_path = os.path.join(
        curPath,'test_data','image', 'test_template.png')
    screen_image = cv2.imread(screen_image_path)
    template_image = cv2.imread(template_image_path)
    return screen_image, template_image

def test_match_template(test_match_setup):
    screen_image, template_image = test_match_setup
    # Set accuracy value
    accuracy_val = 0.9

    # Call the match_template function
    result = match_template(screen_image, template_image, accuracy_val)

    # Check if the result is an instance of ImageRecognitionResult
    assert isinstance(result, ImageRecognitionResult)

    # Check if the result attributes are correctly set
    expected_is_recognized = True
    expected_coordinate = (758, 951)
    expected_recognition_threshold = 0.9999983310699463
    assert result.is_recognized ==  expected_is_recognized
    assert result.coordinate ==  expected_coordinate
    assert result.recognition_threshold == pytest.approx(expected_recognition_threshold, abs=1e-2)


if __name__ == "__main__":
    pytest.main(['-v','-s','pytest_metisse/test_image_recognition.py'])


# Form implementation generated from reading ui file 'c:\Users\henrychen\Documents\gitlab\metisse\metisse\example\ui\universal_script_gui_sub3.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):

    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(1293, 606)
        self.img_label = QtWidgets.QLabel(Dialog)
        self.img_label.setGeometry(QtCore.QRect(10, 10, 1266, 585))
        self.img_label.setObjectName("img_label")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.img_label.setText(_translate("Dialog", "圖片"))



# -*- coding=UTF-8 -*-
# pyright: strict
from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Tuple


class Client(ABC):
    """
    template for client
    """

    @abstractmethod
    def screenshot(self, save_screenshot_path: str) -> None:
        """
        take a screenshot
        """
        ...

    @abstractmethod
    def tap(self, coordinates: Tuple[int, int]) -> None:
        """
        tap device
        """
        ...

    @abstractmethod
    def swipe(self, start_coordinates: Tuple[int, int], end_coordinates: Tuple[int, int], swiping_time: int) -> None:
        """
        swipe device
        """
        ...


class UiSettings(object):
    DEVICE_SCREEN_SIZE = ['1280x720', '720x1280']  #'1488x2266', '1280x720',
    DEVICE_SCREEN_DENSITY = ['160']  # '240'
    LOAD_MAIN_UI = 'universal_script_gui_main.ui'
    LOAD_SUB_UI = ['universal_script_gui_sub.ui','universal_script_gui_sub2.ui','universal_script_gui_sub3.ui']


import logging
import os


class MetisseLogger(logging.Logger):

    def __init__(self, logger_name, log_level=logging.INFO, log_file=None):
        self.logger = logging.getLogger(logger_name)
        self.logger.setLevel(log_level)

        formatter = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )

        console_handler = logging.StreamHandler()
        console_handler.setFormatter(formatter)
        self.logger.addHandler(console_handler)

        if log_file:
            self.set_log_file(log_file, formatter)

    def set_log_file(self, log_file, formatter):
        if not os.path.exists(os.path.dirname(log_file)):
            os.makedirs(os.path.dirname(log_file))

        self.file_handler = logging.FileHandler(log_file)
        self.file_handler.setFormatter(formatter)
        self.logger.addHandler(self.file_handler)

    def close(self):
        if hasattr(self, 'file_handler'):
            self.file_handler.close()
            self.logger.removeHandler(self.file_handler)

    def debug(self, msg, *args, exc_info=None, stack_info=False, stacklevel=1, extra=None):
        self.logger.debug(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)

    def info(self, msg, *args, exc_info=None, stack_info=False, stacklevel=1, extra=None):
        self.logger.info(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)

    def warning(self, msg, *args, exc_info=None, stack_info=False, stacklevel=1, extra=None):
        self.logger.warning(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)

    def error(self, msg, *args, exc_info=None, stack_info=False, stacklevel=1, extra=None):
        self.logger.error(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)

    def critical(self, msg, *args, exc_info=None, stack_info=False, stacklevel=1, extra=None):
        self.logger.critical(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)